package Lib_CGI;

use CGI;
use CGI::Session;
use CGI::Cookie;

# Declaration de vars globales.. ca pue je sais..
my $cgi;
my $session;
my $cookie;
my @cookies_tab;

# Fonction de creation de l'instance de cgi
# Retourne 0 en cas de succes et affiche le
# header d'urgence en cas d'echec..
sub new_cgi
{
	$cgi = CGI->new();

	$| = 1;
	
	send_critical_header("CGI new failed..") if (!defined($cgi));
	return 0;
}

sub print_html_header
{
    if (defined($cgi))
    {
	if (@cookies_tab)
	{
	    print $cgi->header(-type=>'text/html',
			       -cookie=>\@cookies_tab);
	}
	else
	{
	    print $cgi->header('text/html');
	}
	return 0;
    }
    
    # J'ai une erreur de header...
    send_critical_header("CGI ERROR");
}

# Fonction de test si la session en cours est valide ou pas..
# Retourne 0 si ok
# Retourne 1 si non
sub test_session_validity
{
	my $_session = CGI::Session->load()
	or send_critical_header(CGI::Session->errstr);
	
	return 1 if ($_session->is_expired);
	return 0;
}

# Fonction de test si la session est vide
# Retourne 0 si ok
# Retourne 1 si non
sub test_session_empty
{
	my $_session = CGI::Session->load()
	or send_critical_header(CGI::Session->errstr);
	
	return 1 if ($_session->is_empty);
	return 0;
}

# Fonction de test si le visiteur est logge
# retourne 0 si oui
# retourne 1 si jamais
# retourne 2 si expired
sub test_is_logged
{
	return 2 if (test_session_validity());
	return 1 if (test_session_empty());
	return 0;
}

# Fonction de recuperation d'un parametre..
# Prend en parametre le nom du param..
# retourne le parametre ou undef
sub get_param
{
	my $param = shift;
	
	return $cgi->param($param);
}

# Fonction de creation d'une session..
sub new_session
{
	if (!defined($session))
	{
		CGI::Session->name("BamBooWeb");
		$session = new CGI::Session(undef, undef, 
			{Directory=>'/tmp'});
		return 1 if (!defined($session));
		
		$session->expire("1m");
		$session->param("user", "test");
		
		my $_cookie = $cgi->cookie(-name => $session->name, 
			-value  => $session->id );
		push(@cookies_tab, $_cookie);
		
		return 0;
	}
	return 1;
}

# Fonction de recuperation de l'id de la session
sub get_session_id
{
	if (defined($session))
	{
		return $session->id();	
	}
	my $_session = CGI::Session->load()
	or send_critical_header(CGI::Session->errstr);
	return $_session->id();
}

# Fonction de recuperation du remote host
sub get_remote_host
{
	return 1 if (!defined($cgi));
	return $cgi->remote_host();
}

# Fonction de recuperation du remote_host stocke dans une session
sub get_loaded_remote_host
{
	my $_session = CGI::Session->load()
	or send_critical_header(CGI::Session->errstr);
	
	return $_session->remote_addr();
}

# Fonction d'envoie d'un header d'urgence..
# Prend en param un message d'erreur
sub send_critical_header
{
	my $msg = shift;
	
	print <<EOF;
Content-Type: text/html

<html>
<head>
<title>BambooWed Critical Error</title>
</head>
<body>
<center><h1>Error !!</h1></center>
<pre>$msg</pre>
</body>
</html>
EOF
	exit 0;
}

# Fonction de debug... print Dumper
sub debug
{
	use Data::Dumper;
	print "<pre>";
	print Dumper $cgi;
	print "-----------------------\n";
	print Dumper $session;
	print "</pre>";
}

1;
