#!/usr/bin/perl
## ######### PROJECT NAME : ##########
##
## deb_package.pm for
##
## ######### PROJECT DESCRIPTION : ###
##
## Certaines parties de ce code sont pompees du script
## gere_paquet.pm de Romain Dayan (datant de 2006)
##
## ###################################
##
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
##
## Started on  Tue Apr 28 18:05:07 2009 Boutonnet Alexandre
## Last update Thu Apr 30 16:32:18 2009 Boutonnet Alexandre
##
## ###################################
##

package DebPackage;
use strict;
use warnings;

require paths;

# fonction de creation des paquets stables et testing..
sub create_testing_package {
    my ( $category, $svn_repo, $version ) = @_;

    # creation du dossier temporaire
    my $tmp_dir = make_tmp_dir();

    my $repository_path = $PATHS::PROJECT_PATH . $category . "/" . $svn_repo;

    return ("Unable to find $repository_path") if ( !-e $repository_path );

    # Nom du rep ou faire le checkout
    my $svn_repository = $tmp_dir . "/testing-repository";

    # I export svn current directory only

    my $co_ret =
`svn export svn://basson/$category/$svn_repo/version/$version $svn_repository`;

    return (
        "SVN export error : DEBIAN directory of control file not found <br/>")
      if ( test_current_folder($svn_repository) );

    # SVN ne conserve pas les droits, GRRRRR

    foreach ( "postinst", "preinst", "prerm", "postrm" ) {
        chmod 0755, "$svn_repository/DEBIAN/$_";
    }
	
    my $dpkg_ret = `dpkg -b $svn_repository $tmp_dir/deb 2>&1`;

    return ("Package build failed : $dpkg_ret") if ( !`ls $tmp_dir/deb/*` );

    # I get my package architecture type
    my $arch =
`grep Architecture $PATHS::PROJECT_PATH$category\/$svn_repo\/version\/$version\/DEBIAN\/control | cut -f 2 -d \:`;
    $arch =~ s/\s//g;

    my $package_path = `echo $tmp_dir/deb/*`;
    chomp $package_path;

    # I move my debian package in the repository
    my $ret_move =
      move_deb_in_repository( $category, "testing", $arch, $package_path );
    return ("Error in moving package in repository : $ret_move")
      if ($ret_move);

    # I make the package.gz...
    #my $ret_pack = create_packagegz( $category, "testing", $arch );
    #return ("Error in Packages.gz creation : $ret_pack") if ($ret_pack);

    return (0);
}

# fonction de creation des paquets unstables..
sub create_unstable_package {
    my ( $category, $svn_repo ) = @_;

    # creation du dossier temporaire
    my $tmp_dir = make_tmp_dir();

    my $repository_path = $PATHS::PROJECT_PATH . $category . "/" . $svn_repo;

    return ("Unable to find $repository_path") if ( !-e $repository_path );

    # Nom du rep ou faire le checkout
    my $svn_repository = $tmp_dir . "/unstable-repository";

    # I export svn current directory only
    my $co_ret =
      `svn export svn://basson/$category/$svn_repo/current $svn_repository`;

    return (
        "SVN export error : DEBIAN directory of control file not found <br/>")
      if ( test_current_folder($svn_repository) );

    my $dpkg_ret = `dpkg -b $svn_repository $tmp_dir/deb 2>&1`;

    return ("Package build failed: $dpkg_ret") if ( !`ls $tmp_dir/deb/*` );

    # I get my package architecture type
    my $arch =
`grep Architecture $PATHS::PROJECT_PATH$category\/$svn_repo\/current\/DEBIAN\/control | cut -f 2 -d \:`;
    $arch =~ s/\s//g;

    my $package_path = `echo $tmp_dir/deb/*`;
    chomp $package_path;

    # I move my debian package in the repository
    my $ret_move =
      move_deb_in_repository( $category, "unstable", $arch, $package_path );
    return ("Error in moving package in repository : $ret_move")
      if ($ret_move);

    # I make the package.gz...
    #my $ret_pack = create_packagegz( $category, "unstable", $arch );
    #return ("Error in Packages.gz creation : $ret_pack") if ($ret_pack);

    return (0);
}

# This function makes a new branch :
# - copying the current folder to a version folder
# - making an commit
sub create_branch {
    my ( $category, $svn_repo ) = @_;

    my $svn_repository = "$PATHS::PROJECT_PATH$category/$svn_repo";

    # First of all I have to make an update
    my $cmd_ret = `svn up $svn_repository`;

    return (
"SVN update error : DEBIAN directory of control file not found in current folder<br/>"
    ) if ( test_current_folder("$svn_repository/current") );

    # getting current version
    my $version =
      `grep Version $svn_repository/current\/DEBIAN\/control | cut -f 2 -d \:`;
    chomp $version;
    $version =~ s/\s//g;

    return ("Unable to get version") if ( !$version );
	
	# bug #439
	return ("version $version already exists!") if ( -d "$svn_repository/version/$version");
	
    # Making svn copy
    $cmd_ret =
`svn copy --username webminsvn --password spider77  $svn_repository/current $svn_repository/version/$version`;

    return (
"SVN copy error : DEBIAN directory of control file not found in version/$version folder<br/>"
    ) if ( test_current_folder("$svn_repository/version/$version") );

    $cmd_ret =
`svn commit --non-interactive --username webminsvn --password spider77 -m \"version $version created\" $svn_repository/ 2>&1`;

    return (0);
}

# move debian package in the repository folder..
# With some tests..
# 20090429 : arch special case is not implemented
# 20090512 : repository directory tree changed
sub move_deb_in_repository {
    my ( $category, $type, $arch, $package_path ) = @_;

    my $command;

    `mkdir -p $PATHS::REPOSITORY_PATH/pool/$category/$type/$arch/`
      if ( !-d "$PATHS::REPOSITORY_PATH/pool/$category/$type/$arch/" );
    $command =
      "cp $package_path $PATHS::REPOSITORY_PATH/pool/$category/$type/$arch/";
    return `$command 2>&1`;
}

# creer un dossier dans le dossier temporaire de la machine
# Renvoie le nom de ce dossier
sub make_tmp_dir {
    my $rep = "/tmp/debian_package_" . $$ . "/";

    del_tmp_dir();

    # creation du repertoire temporaire
    # Creation du dossier d'envoie des .deb
    `mkdir -p $rep/deb`;

    return $rep;
}

sub del_tmp_dir {
    my $rep = "/tmp/debian_package_" . $$ . "/";

    # supression du repertoire si il existe deja
    `rm -fr $rep`;
}

# Verifie la presence du dossier DEBIAN contenant un fichier control
sub test_current_folder {
    my $folder = shift;
    my $error  = 0;

    $folder .= "/";

    $error += 1 unless ( -d $folder . "DEBIAN" );
    $error += 2 unless ( -f $folder . "DEBIAN/control" );
	print "<pre> $folder </pre>";
    return $error;
}

# Creer le package.gz
sub create_packagegz {
    my ( $category, $type, $arch ) = @_;

    my $ret_value;

    # je ne vais pas me prendre la tete, je vais
    # regenerer tous les packages.gz..
    # Si c'est genant (trop long) faudra changer

    my $tmp_pool_path  = "$PATHS::REPOSITORY_PATH/pool/";
    my $catego_st_path = "$PATHS::REPOSITORY_PATH/pool/*/";

    my @catego_path = split( / /, `echo $catego_st_path` );

    foreach my $catego_repo_path (@catego_path) {
        chomp $catego_repo_path;

        my ($tmp_catego) = ( $catego_repo_path =~ m/$tmp_pool_path(.+)\// );

        next if ( $tmp_catego eq "intliq" );

        my $type_st_path = $tmp_pool_path . $tmp_catego . "/*/";
        my @type_path = split( / /, `echo $type_st_path` );

        foreach my $type_repo_path (@type_path) {
            chomp $type_repo_path;

            my ($tmp_type) = ( $type_repo_path =~ m/$tmp_pool_path.+\/(.+)\// );

            my $arch_st_path =
              $tmp_pool_path . $tmp_catego . "/" . $tmp_type . "/*/";

            my @arch_path = split( / /, `echo $arch_st_path` );

            foreach my $arch_repo_path (@arch_path) {
                chomp $arch_repo_path;

                my ($tmp_arch) =
                  ( $arch_repo_path =~ m/$tmp_pool_path.+\/.+\/(.+)\// );

                next if ( $tmp_arch eq "all" );

                my $package_tmp_path_file =
                    "/tmp/" 
                  . $$
                  . "_package_"
                  . $tmp_catego . "_"
                  . $tmp_type . "_"
                  . $tmp_arch;

`mkdir -p $PATHS::REPOSITORY_PATH/dists/$tmp_catego/$tmp_type/binary-$tmp_arch/`
                  if (
                    !-d "$PATHS::REPOSITORY_PATH/dists/$tmp_catego/$tmp_type/binary-$tmp_arch/"
                  );

                my $command = "cd $PATHS::REPOSITORY_PATH";
                $command .=
" && dpkg-scanpackages pool/$tmp_catego/$tmp_type/$tmp_arch /dev/null > $package_tmp_path_file";
                $command .=
" ; dpkg-scanpackages pool/$tmp_catego/$tmp_type/all /dev/null >> $package_tmp_path_file";
                $command .=
" ; dpkg-scanpackages pool/intliq/$tmp_type/$tmp_arch /dev/null >> $package_tmp_path_file";
                $command .=
" ; dpkg-scanpackages pool/intliq/$tmp_type/all /dev/null >> $package_tmp_path_file";
                $command .=
" ; cat $package_tmp_path_file | gzip -f9 > dists/$tmp_catego/$tmp_type/binary-$tmp_arch/Packages.gz";
                $command .=
" ; cp $package_tmp_path_file dists/$tmp_catego/$tmp_type/binary-$tmp_arch/Packages";

                $ret_value .= `$command`;
            }

        }
    }

    return ($ret_value);
}

1;
